# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午3:59
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : output.py
@Software: PyCharm
"""
import os.path
import traceback
from typing import Optional

from litedoc.style.markdown import generate
from litedoc.syntax.astparser import AstParser


def write_to_file(content: str, output: str) -> None:
    """
    Write content to file.

    Args:
        content: str, content to write.
        output: str, path to output file.
    """
    if not os.path.exists(os.path.dirname(output)):
        os.makedirs(os.path.dirname(output))

    with open(output, "w", encoding="utf-8") as f:
        f.write(content)


def get_file_list(module_folder: str):
    file_list = []
    for root, dirs, files in os.walk(module_folder):
        for file in files:
            if file.endswith((".py", ".pyi")):
                file_list.append(os.path.join(root, file))
    return file_list


def get_relative_path(base_path: str, target_path: str) -> str:
    """
    获取相对路径
    Args:
        base_path: 基础路径
        target_path: 目标路径
    """
    return os.path.relpath(target_path, base_path)


def generate_from_module(module_folder: str,
                         output_dir: str,
                         with_top: bool = False,
                         lang: str = "zh-Hans",
                         ignored_paths=None,
                         theme: str = "vitepress",
                         style: str = "google",
                         frontmatter: Optional[dict] = None,
                         **kwargs
                         ):
    """
    生成文档
    Args:
        module_folder: 模块文件夹
        output_dir: 输出文件夹
        with_top: 是否包含顶层文件夹 False时例如docs/api/module_a, docs/api/module_b， True时例如docs/api/module/module_a.md， docs/api/module/module_b.md
        ignored_paths: 忽略的路径
        lang: 语言
        theme: 主题
        style: 样式
        frontmatter:
    """
    if ignored_paths is None:
        ignored_paths = []
    file_data: dict[str, str] = {}  # 路径 -> 字串

    file_list = get_file_list(module_folder)

    # 清理输出目录
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    replace_data = {
            "__init__": "index" if theme == "vitepress" else "README",
            ".py"     : ".md",
    }

    total_file_count = len(file_list)
    generate_file_count = 0
    for pyfile_path in file_list:
        try:
            if any(ignored_path.replace("\\", "/") in pyfile_path.replace("\\", "/") for ignored_path in ignored_paths):
                continue
            no_module_name_pyfile_path = get_relative_path(module_folder, pyfile_path)  # 去头路径
            # markdown相对路径
            rel_md_path = pyfile_path if with_top else no_module_name_pyfile_path
            for rk, rv in replace_data.items():
                rel_md_path = rel_md_path.replace(rk, rv)
            base_name = os.path.basename(rel_md_path)  # index.md
            abs_md_path = os.path.join(output_dir, rel_md_path)  # 最终输出路径

            create_same_path = os.path.join(os.path.dirname(abs_md_path), os.path.basename(os.path.dirname(abs_md_path))) + ".md"

            title = (pyfile_path.replace("\\", "/")
                     .replace("/", ".")
                     .replace(".py", "")
                     .replace(".__init__", ""))
            # 获取模块信息
            ast_parser = AstParser(open(pyfile_path, "r", encoding="utf-8").read(), title=title, style=style, file_path=no_module_name_pyfile_path)
            # 生成markdown
            config_front_matter = {
                    "title": title,
            }

            if frontmatter is not None:
                config_front_matter.update(frontmatter)

            if base_name == "index.md":
                config_front_matter["collapsed"] = "true"

            md_content = generate(ast_parser, lang=lang, frontmatter=config_front_matter, **kwargs)
            file_data[abs_md_path] = md_content
            if kwargs.get("cs", False) and base_name == "index.md":
                file_data[create_same_path] = md_content
            print(f"Output {pyfile_path} -> {abs_md_path}")
            generate_file_count += 1
        except Exception as e:
            print(f"Error in {pyfile_path}: {e}")
            traceback.print_exc()

    for fn, content in file_data.items():
        write_to_file(content, fn)
    print(f"\nComplete:    {generate_file_count}/{total_file_count} success    {total_file_count - generate_file_count} failed\n")



# -*- coding: utf-8 -*-
"""
本模块定义了一些数学方程相关的函数。
"""

from mbcp.mp_math.mp_math_typing import OneVarFunc, Var, MultiVarsFunc, Number
from mbcp.mp_math.point import Point3
from mbcp.mp_math.const import EPSILON


def cal_gradient_3vf(func: ThreeSingleVarsFunc, p: Point3, epsilon: float = EPSILON) -> Vector3:
    """
    计算三元函数在某点的梯度向量。
    $\nabla f(x_0, y_0, z_0) = \\left(\\frac{\\partial f}{\\partial x}, \\frac{\\partial f}{\\partial y}, \\frac{\\partial f}{\\partial z}\\right)$
    Args:
        func: 三元函数
        p: 点
        epsilon: 偏移量
    Returns:
        梯度
    """
    dx = (func(p.x + epsilon, p.y, p.z) - func(p.x - epsilon, p.y, p.z)) / (2 * epsilon)
    dy = (func(p.x, p.y + epsilon, p.z) - func(p.x, p.y - epsilon, p.z)) / (2 * epsilon)
    dz = (func(p.x, p.y, p.z + epsilon) - func(p.x, p.y, p.z - epsilon)) / (2 * epsilon)
    return Vector3(dx, dy, dz)


def cal_gradient_3vf2(func: ThreeSingleVarsFunc, p: Point3, epsilon: float = EPSILON) -> Vector3:
    """
    计算三元函数在某点的梯度向量。
    > [!tip]
    > 已知一个函数$f(x, y, z)$，则其在点$(x_0, y_0, z_0)$处的梯度向量为:
    Args:
        func: 三元函数
        p: 点
        epsilon: 偏移量
    Returns:
        梯度
    """
    dx = (func(p.x + epsilon, p.y, p.z) - func(p.x - epsilon, p.y, p.z)) / (2 * epsilon)
    dy = (func(p.x, p.y + epsilon, p.z) - func(p.x, p.y - epsilon, p.z)) / (2 * epsilon)
    dz = (func(p.x, p.y, p.z + epsilon) - func(p.x, p.y, p.z - epsilon)) / (2 * epsilon)
    return Vector3(dx, dy, dz)


class CurveEquation:
    def __init__(self, x_func: OneVarFunc, y_func: OneVarFunc, z_func: OneVarFunc):
        """
        曲线方程。
        Args:
            x_func: x函数
            y_func: y函数
            z_func: z函数
        """
        self.x_func = x_func
        self.y_func = y_func
        self.z_func = z_func

    def __call__(self, *t: Var) -> Point3 | tuple[Point3, ...]:
        """
        计算曲线上的点。
        Args:
            *t:
                参数
        Returns:

        """
        if len(t) == 1:
            return Point3(self.x_func(t[0]), self.y_func(t[0]), self.z_func(t[0]))
        else:
            return tuple([Point3(x, y, z) for x, y, z in zip(self.x_func(t), self.y_func(t), self.z_func(t))])

    def __str__(self):
        return "CurveEquation()"


def get_partial_derivative_func(func: MultiVarsFunc, var: int | tuple[int, ...], epsilon: Number = EPSILON) -> MultiVarsFunc:
    """
    求N元函数一阶偏导函数。这玩意不太稳定，慎用。
    Args:
        func: 函数
        var: 变量位置，可为整数(一阶偏导)或整数元组(高阶偏导)
        epsilon: 偏移量
    Returns:
        偏导函数
    Raises:
        ValueError: 无效变量类型
    """
    if isinstance(var, int):
        def partial_derivative_func(*args: Var) -> Var:
            """
            :param args:
            :return:
            """
            args_list_plus = list(args)
            args_list_plus[var] += epsilon
            args_list_minus = list(args)
            args_list_minus[var] -= epsilon
            return (func(*args_list_plus) - func(*args_list_minus)) / (2 * epsilon)

        return partial_derivative_func
    elif isinstance(var, tuple):
        def high_order_partial_derivative_func(*args: Var) -> Var:
            """
            求高阶偏导函数 @litedoc-hide
            :param args:
            :return:
            """
            result_func = func
            for v in var:
                result_func = get_partial_derivative_func(result_func, v, epsilon)
            return result_func(*args)

        return high_order_partial_derivative_func
    else:
        raise ValueError("Invalid var type")


def curry(func: MultiVarsFunc, *args: Var) -> OneVarFunc:
    """
    对多参数函数进行柯里化。
    > [!tip]
    > 有关函数柯里化，可参考[函数式编程--柯理化（Currying）](https://zhuanlan.zhihu.com/p/355859667)
    Args:
        func: 函数
        *args: 参数
    Returns:
        柯里化后的函数
    Examples:
        ```python
        def add(a: int, b: int, c: int) -> int:
            return a + b + c
        add_curried = curry(add, 1, 2)
        add_curried(3)  # 6
        ```
    """

    def curried_func(*args2: Var) -> Var:
        """@litedoc-hide"""
        return func(*args, *args2)

    return curried_func


def test_kwargs(*args, **kwargs):
    """
    测试kwargs
    Args:
        *args:
        **kwargs:
    """
    print(args)
    print(kwargs)



from typing import TYPE_CHECKING, overload

from .const import APPROX
from .utils import approx

if TYPE_CHECKING:
    from .vector import Vector3  # type: ignore


class Point3:
    def __init__(self, x: float, y: float, z: float):
        """
        笛卡尔坐标系中的点。
        Args:
            x: x 坐标
            y: y 坐标
            z: z 坐标
        """
        self.x = x
        self.y = y
        self.z = z

    def approx(self, other: "Point3", epsilon: float = APPROX) -> bool:
        """
        判断两个点是否近似相等。
        Args:
            other:
            epsilon:

        Returns:
            是否近似相等
        """
        return all([abs(self.x - other.x) < epsilon, abs(self.y - other.y) < epsilon, abs(self.z - other.z) < epsilon])

    def __str__(self):
        return f"Point3({self.x}, {self.y}, {self.z})"

    @overload
    def __add__(self, other: "Vector3") -> "Point3":
        ...

    @overload
    def __add__(self, other: "Point3") -> "Point3":
        ...

    def __add__(self, other):
        """
        P + V -> P
        P + P -> P
        Args:
            other:
        Returns:
        """
        return Point3(self.x + other.x, self.y + other.y, self.z + other.z)

    def __eq__(self, other):
        """
        判断两个点是否相等。
        Args:
            other:
        Returns:
        """
        return approx(self.x, other.x) and approx(self.y, other.y) and approx(self.z, other.z)

    def __sub__(self, other: "Point3") -> "Vector3":
        """
        P - P -> V

        P - V -> P  已在 :class:`Vector3` 中实现
        Args:
            other:
        Returns:

        """
        from .vector import Vector3
        return Vector3(self.x - other.x, self.y - other.y, self.z - other.z)



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午1:46
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : __init__.py.py
@Software: PyCharm
"""


# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/6 下午12:57
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : other.py
@Software: PyCharm
"""
import math
from typing import TYPE_CHECKING

from .const import APPROX
from .mp_math_typing import OneSingleVarFunc, RealNumber
from .utils import sign_format
from .vector import Vector3

if TYPE_CHECKING:
    from .angle import AnyAngle
    from .point import Point3


class Line3:
    def __init__(self, point: 'Point3', direction: 'Vector3'):
        """
        三维空间中的直线。由一个点和一个方向向量确定。
        Args:
            point: 直线上的一点
            direction: 直线的方向向量
        """
        self.point = point
        self.direction = direction

    def approx(self, other: 'Line3', epsilon: float = APPROX) -> bool:
        """
        判断两条直线是否近似相等。
        Args:
            other: 另一条直线
            epsilon: 误差
        Returns:
            是否近似相等
        """
        return self.is_approx_parallel(other, epsilon) and (self.point - other.point).is_approx_parallel(self.direction, epsilon)

    def cal_angle(self, other: 'Line3') -> 'AnyAngle':
        """
        计算直线和直线之间的夹角。
        Args:
            other: 另一条直线
        Returns:
            夹角弧度
        Raises:
            TypeError: 不支持的类型
        """
        return self.direction.cal_angle(other.direction)

    def cal_distance(self, other: 'Line3 | Point3') -> float:
        """
        计算直线和直线或点之间的距离。
        Args:
            other: 平行直线或点

        Returns:
            距离
        Raises:
            TypeError: 不支持的类型
        """
        if isinstance(other, Line3):
            # 相交/重合 = 0；平行和异面需要计算距离
            if self == other:
                # 重合
                return 0
            elif self.is_parallel(other):
                # 平行
                return (other.point - self.point).cross(self.direction).length / self.direction.length
            elif not self.is_coplanar(other):
                # 异面
                return abs(self.direction.cross(other.direction) @ (self.point - other.point) / self.direction.cross(other.direction).length)
            else:
                # 相交
                return 0

        elif isinstance(other, Point3):
            return (other - self.point).cross(self.direction).length / self.direction.length
        else:
            raise TypeError("Unsupported type.")

    def cal_intersection(self, other: 'Line3') -> 'Point3':
        """
        计算两条直线的交点。
        Args:
            other: 另一条直线
        Returns:
            交点
        Raises:
            ValueError: 直线平行
            ValueError: 直线不共面
        """
        if self.is_parallel(other):
            raise ValueError("Lines are parallel and do not intersect.")
        if not self.is_coplanar(other):
            raise ValueError("Lines are not coplanar and do not intersect.")
        return (self.point + (self.direction.cross(other.direction) @ other.direction.cross(self.point - other.point)) /
                self.direction.cross(other.direction).length ** 2 * self.direction)

    def cal_perpendicular(self, point: 'Point3') -> 'Line3':
        """
        计算直线经过指定点p的垂线。
        Args:
            point: 指定点
        Returns:
            垂线
        """
        return Line3(point, self.direction.cross(point - self.point))

    def get_point(self, t: RealNumber) -> 'Point3':
        """
        获取直线上的点。同一条直线，但起始点和方向向量不同，则同一个t对应的点不同。
        Args:
            t: 参数t
        Returns:
            点
        """
        return self.point + t * self.direction

    def get_parametric_equations(self) -> tuple[OneSingleVarFunc, OneSingleVarFunc, OneSingleVarFunc]:
        """
        获取直线的参数方程。
        Returns:
            x(t), y(t), z(t)
        """
        return (lambda t: self.point.x + self.direction.x * t,
                lambda t: self.point.y + self.direction.y * t,
                lambda t: self.point.z + self.direction.z * t)

    def is_approx_parallel(self, other: 'Line3', epsilon: float = 1e-6) -> bool:
        """
        判断两条直线是否近似平行。
        Args:
            other: 另一条直线
            epsilon: 误差
        Returns:
            是否近似平行
        """
        return self.direction.is_approx_parallel(other.direction, epsilon)

    def is_parallel(self, other: 'Line3') -> bool:
        """
        判断两条直线是否平行。
        Args:
            other: 另一条直线
        Returns:
            是否平行
        """
        return self.direction.is_parallel(other.direction)

    def is_collinear(self, other: 'Line3') -> bool:
        """
        判断两条直线是否共线。
        Args:
            other: 另一条直线
        Returns:
            是否共线
        """
        return self.is_parallel(other) and (self.point - other.point).is_parallel(self.direction)

    def is_point_on(self, point: 'Point3') -> bool:
        """
        判断点是否在直线上。
        Args:
            point: 点
        Returns:
            是否在直线上
        """
        return (point - self.point).is_parallel(self.direction)

    def is_coplanar(self, other: 'Line3') -> bool:
        """
        判断两条直线是否共面。
        充要条件：两直线方向向量的叉乘与两直线上任意一点的向量的点积为0。
        Args:
            other: 另一条直线
        Returns:
            是否共面
        """
        return self.direction.cross(other.direction) @ (self.point - other.point) == 0

    def simplify(self):
        """
        简化直线方程，等价相等。
        自体简化，不返回值。

        按照可行性一次对x y z 化 0 处理，并对向量单位化
        """
        self.direction.normalize()
        # 平行与zy平面，x始终为0
        if self.direction.x == 0:
            self.point.x = 0
        # 平行与xz平面，y始终为0
        if self.direction.y == 0:
            self.point.y = 0
        # 平行与xy平面，z始终为0
        if self.direction.z == 0:
            self.point.z = 0

    @classmethod
    def from_two_points(cls, p1: 'Point3', p2: 'Point3') -> 'Line3':
        """
        工厂函数 由两点构造直线。
        Args:
            p1: 点1
            p2: 点2
        Returns:
            直线
        """
        direction = p2 - p1
        return cls(p1, direction)

    def __and__(self, other: 'Line3') -> 'Line3 | Point3 | None':
        """
        计算两条直线点集合的交集。重合线返回自身，平行线返回None，交线返回交点。
        Args:
            other: 另一条直线
        Returns:
            交点
        """
        if self.is_collinear(other):
            return self
        elif self.is_parallel(other) or not self.is_coplanar(other):
            return None
        else:
            return self.cal_intersection(other)

    def __eq__(self, other) -> bool:
        """
        判断两条直线是否等价。

        v1 // v2 ∧ (p1 - p2) // v1
        Args:
            other:

        Returns:

        """
        return self.direction.is_parallel(other.direction) and (self.point - other.point).is_parallel(self.direction)

    def __str__(self):
        """
        返回点向式（x-x0）
        Returns:

        """
        s = "Line3: "
        if self.direction.x != 0:
            s += f"(x{sign_format(-self.point.x)})/{self.direction.x}"
        if self.direction.y != 0:
            s += f" = (y{sign_format(-self.point.y)})/{self.direction.y}"
        if self.direction.z != 0:
            s += f" = (z{sign_format(-self.point.z)})/{self.direction.z}"
        return s

    def __repr__(self):
        return f"Line3({self.point}, {self.direction})"



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午4:08
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : __main__.py
@Software: PyCharm
"""
# command line tool
# args[0] path
# -o|--output output path
# -l|--lang zh-Hans en jp default zh-Hans


import argparse
import os
import sys

from litedoc.output import generate_from_module


def main():
    parser = argparse.ArgumentParser(description="Generate documentation from Python modules.")
    parser.add_argument("path", type=str, help="Path to the Python module or package.")
    parser.add_argument("-o", "--output", default="doc-output", type=str, help="Output directory.")
    parser.add_argument("-c", "--contain-top", action="store_true", help="Whether to contain top-level dir in output dir.")
    parser.add_argument("-cs", "--create-same", action="store_true", help="Create same file with folder name")
    parser.add_argument("-bu", "--base-url", default=None, type=str, help="base url of the document.")
    parser.add_argument("-l", "--lang", default="zh_Hans", type=str, help="Languages of the document.")
    parser.add_argument("-t", "--theme", default="vitepress", type=str, help="Theme of the document.")
    parser.add_argument("-s", "--style", default="google", type=str, help="Style of the document.")
    parser.add_argument("-f", "--frontmatter", default=None, type=str, help="Frontmatter of the document.")

    parser.add_argument("-fd", "--function-define", default="func", type=str, help="Function define of the document.")
    parser.add_argument("-md", "--method-define", default="method", type=str, help="Class function define of the document.")
    parser.add_argument("-cd", "--class-define", default="class", type=str, help="Class define of the document.")
    parser.add_argument("-vd", "--var-define", default="var", type=str, help="Variable define of the document.")
    parser.add_argument("-ad", "--attr-define", default="attr", type=str, help="Attribute define of the document.")
    # frontmatter 输入格式为 key1=value1,key2=value2, 空格用%20代替

    args = parser.parse_args()

    if not os.path.exists(args.path):
        print(f"Error: The path {args.path} does not exist.")
        sys.exit(1)

    if not os.path.exists(args.output):
        os.makedirs(args.output)

    lang = args.lang

    if args.frontmatter is not None:
        frontmatter = {}
        for item in args.frontmatter.split(","):
            key, value = item.split("=")
            frontmatter[key] = value.replace("%20", " ")
    else:
        frontmatter = None

    generate_from_module(
        args.path, args.output,
        with_top=args.contain_top,
        lang=lang,
        theme=args.theme,
        style=args.style,
        frontmatter=frontmatter,
        fd=args.function_define,
        md=args.method_define,
        cd=args.class_define,
        vd=args.var_define,
        ad=args.attr_define,
        cs=args.create_same,
        bu=args.base_url
    )


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/29 下午2:36
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : version.py.py
@Software: PyCharm
"""


def format_date_number() -> str:
    """
    将日期转换为20240212114514
    Args:
        date: 日期
    Returns:
        日期字符串
    """
    import datetime
    date = datetime.datetime.now()
    return date.strftime("%Y%m%d%H%M%S")


def get_version() -> str:
    return f"0.1.0.dev{format_date_number()}"



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/12 下午9:16
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : utils.py
@Software: PyCharm
"""
from typing import overload, TYPE_CHECKING

from .mp_math_typing import RealNumber
from .const import APPROX

if TYPE_CHECKING:
    from .vector import Vector3
    from .point import Point3
    from .plane import Plane3
    from .line import Line3


def clamp(x: float, min_: float, max_: float) -> float:
    """
    区间限定函数
    Args:
        x: 待限定的值
        min_: 最小值
        max_: 最大值

    Returns:
        限制后的值
    """
    return max(min(x, max_), min_)


class Approx:
    """
    用于近似比较对象

    已实现对象 实数 Vector3 Point3 Plane3 Line3
    """
    def __init__(self, value: RealNumber):
        self.value = value

    def __eq__(self, other):
        if isinstance(self.value, (float, int)):
            if isinstance(other, (float, int)):
                return abs(self.value - other) < APPROX
            else:
                self.raise_type_error(other)
        elif isinstance(self.value, Vector3):
            if isinstance(other, (Vector3, Point3, Plane3, Line3)):
                return all([approx(self.value.x, other.x), approx(self.value.y, other.y), approx(self.value.z, other.z)])
            else:
                self.raise_type_error(other)

    def raise_type_error(self, other):
        raise TypeError(f"Unsupported type: {type(self.value)} and {type(other)}")

    def __ne__(self, other):
        return not self.__eq__(other)


def approx(x: float, y: float = 0.0, epsilon: float = APPROX) -> bool:
    """
    判断两个数是否近似相等。或包装一个实数，用于判断是否近似于0。
    Args:
        x: 数1
        y: 数2
        epsilon: 误差
    Returns:
        是否近似相等
    """
    return abs(x - y) < epsilon


def sign(x: float, only_neg: bool = False) -> str:
    """获取数的符号。
    Args:
        x: 数
        only_neg: 是否只返回负数的符号
    Returns:
        符号 + - ""
    """
    if x > 0:
        return "+" if not only_neg else ""
    elif x < 0:
        return "-"
    else:
        return ""


def sign_format(x: float, only_neg: bool = False) -> str:
    """格式化符号数
    -1 -> -1
    1 -> +1
    0 -> ""
    Args:
        x: 数
        only_neg: 是否只返回负数的符号
    Returns:
        符号 + - ""
    """
    if x > 0:
        return f"+{x}" if not only_neg else f"{x}"
    elif x < 0:
        return f"-{abs(x)}"
    else:
        return ""



"""
Google docstring parser for Python.
"""
from typing import Optional

from litedoc.docstring.docstring import Docstring


class Parser:
    ...


class GoogleDocstringParser(Parser):
    _tokens = {
            "Args"       : "args",
            "Arguments"  : "args",
            "参数"       : "args",

            "Return"     : "return",
            "Returns"    : "return",
            "返回"       : "return",

            "Attribute"  : "attribute",
            "Attributes" : "attribute",
            "属性"       : "attribute",

            "Raises"     : "raises",
            "Raise"      : "raises",
            "引发"       : "raises",

            "Example"    : "example",
            "Examples"   : "example",
            "示例"       : "example",

            "Yields"     : "yields",
            "Yield"      : "yields",
            "产出"       : "yields",

            "Requires"   : "requires",
            "Require"    : "requires",
            "需要"       : "requires",

            "FrontMatter": "front_matter",
            "前言"       : "front_matter",
    }

    def __init__(self, docstring: str, indent: int = 4):
        self.lines = docstring.splitlines()
        self.indent = indent
        self.lineno = 0  # Current line number
        self.char = 0  # Current character position

        self.docstring = Docstring(raw=docstring)

    def read_line(self, move: bool = True) -> str:
        """
        每次读取一行
        Args:
            move: 是否移动指针
        Returns:
        """
        if self.lineno >= len(self.lines):
            return ""
        line = self.lines[self.lineno]
        if move:
            self.lineno += 1
        return line

    def match_token(self) -> Optional[str]:
        """
        解析下一行的token
        Returns:

        """
        for token in self._tokens:
            line = self.read_line(move=False)
            if line.strip().startswith(token):
                self.lineno += 1
                return self._tokens[token]
        return None

    def parse_args(self):
        """
        依次解析后面的参数行，直到缩进小于等于当前行的缩进
        """
        while line := self.match_next_line():
            if ":" in line:
                name, desc = line.split(":", 1)
                self.docstring.add_arg(name.strip(), desc.strip())
            else:
                self.docstring.add_arg(line.strip())

    def parse_return(self):
        """
        解析返回值行
        """
        if line := self.match_next_line():
            self.docstring.add_return(line.strip())

    def parse_raises(self):
        """
        解析异常行
        """
        while line := self.match_next_line():
            if ":" in line:
                name, desc = line.split(":", 1)
                self.docstring.add_raise(name.strip(), desc.strip())
            else:
                self.docstring.add_raise(line.strip())

    def parse_example(self):
        """
        解析示例行
        """
        while line := self.match_next_line():
            self.docstring.add_example(line if line.startswith("    ") else line.strip())

    def parse_attrs(self):
        """
        解析属性行
        """
        while line := self.match_next_line():
            if ":" in line:
                name, desc = line.split(":", 1)
                self.docstring.add_attrs(name.strip(), desc.strip())
            else:
                self.docstring.add_attrs(line.strip())

    def match_next_line(self) -> Optional[str]:
        """
        在一个子解析器中，解析下一行，直到缩进小于等于当前行的缩进
        Returns:
        """
        line = self.read_line(move=False)
        if line.startswith(" " * self.indent):
            self.lineno += 1
            return line[self.indent:]
        else:
            return None

    def parse(self) -> Docstring:
        """
        逐行解析，直到遇到EOS

        最开始未解析到的内容全部加入desc

        Returns:

        """
        add_desc = True
        while self.lineno < len(self.lines):
            token = self.match_token()
            if token is None and add_desc:
                self.docstring.add_desc(self.lines[self.lineno].strip())

            if token is not None:
                add_desc = False

            match token:
                case "args":
                    self.parse_args()
                case "return":
                    self.parse_return()
                case "attribute":
                    self.parse_attrs()
                case "raises":
                    self.parse_raises()
                case "example":
                    self.parse_example()
                case _:
                    self.lineno += 1

        return self.docstring


class NumpyDocstringParser(Parser):
    ...


class ReStructuredParser(Parser):
    ...


def parse(docstring: str, parser: str = "google", indent: int = 4) -> Docstring:
    if parser == "google":
        return GoogleDocstringParser(docstring, indent).parse()
    else:
        raise ValueError(f"Unknown parser: {parser}")



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/9 上午11:35
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : mp_math_typing.py
@Software: PyCharm
"""
from typing import Callable, Iterable, TypeAlias, TypeVar

RealNumber: TypeAlias = int | float  # 实数@litedoc-hide
Number: TypeAlias = RealNumber | complex  # 数
SingleVar = TypeVar("SingleVar", bound=Number)  # 单变量
ArrayVar = TypeVar("ArrayVar", bound=Iterable[Number])  # 数组变量
Var: TypeAlias = SingleVar | ArrayVar  # 变量

OneSingleVarFunc: TypeAlias = Callable[[SingleVar], SingleVar]  # 一元单变量函数
OneArrayFunc: TypeAlias = Callable[[ArrayVar], ArrayVar]  # 一元数组函数
OneVarFunc: TypeAlias = OneSingleVarFunc | OneArrayFunc  # 一元函数

TwoSingleVarsFunc: TypeAlias = Callable[[SingleVar, SingleVar], SingleVar]  # 二元单变量函数
TwoArraysFunc: TypeAlias = Callable[[ArrayVar, ArrayVar], ArrayVar]  # 二元数组函数
TwoVarsFunc: TypeAlias = TwoSingleVarsFunc | TwoArraysFunc  # 二元函数

ThreeSingleVarsFunc: TypeAlias = Callable[[SingleVar, SingleVar, SingleVar], SingleVar]  # 三元单变量函数
ThreeArraysFunc: TypeAlias = Callable[[ArrayVar, ArrayVar, ArrayVar], ArrayVar]  # 三元数组函数
ThreeVarsFunc: TypeAlias = ThreeSingleVarsFunc | ThreeArraysFunc  # 三元函数

MultiSingleVarsFunc: TypeAlias = Callable[..., SingleVar]  # 多元单变量函数
MultiArraysFunc: TypeAlias = Callable[..., ArrayVar]  # 多元数组函数
MultiVarsFunc: TypeAlias = MultiSingleVarsFunc | MultiArraysFunc  # 多元函数



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午3:39
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : markdown.py
@Software: PyCharm
"""
from typing import Optional

from litedoc.syntax.astparser import AstParser
from litedoc.syntax.node import *
from litedoc.i18n import get_text

litedoc_hide = "@litedoc-hide"


def generate(parser: AstParser, lang: str, frontmatter: Optional[dict] = None, **kwargs) -> str:
    """
    Generate markdown style document from ast
    You can modify this function to generate markdown style that enjoys you
    Args:
        parser:
        lang: language
        frontmatter:
        **kwargs: 更多参数
    Returns:
        markdown style document
    """
    if frontmatter is not None:
        md = "---\n"
        for k, v in frontmatter.items():
            md += f"{k}: {v}\n"
        md += "---\n"
    else:
        md = ""

    # 添加标题，如果有
    if parser.title is not None:
        md += f"# {parser.title}\n\n"

    # 添加描述，如果有
    if parser.description is not None:
        md += f"{parser.description.markdown(lang)}\n\n"

    for node in parser.all_nodes:
        if isinstance(node, FunctionNode):
            if node.name.startswith("_") or (node.docs is not None and litedoc_hide in node.docs.reduction()):
                continue
            md += node.markdown(lang, **kwargs)
        elif isinstance(node, ClassNode):
            md += node.markdown(lang, **kwargs)
        elif isinstance(node, AssignNode):
            if node.docs is not None and litedoc_hide not in node.docs:
                md += node.markdown(lang, **kwargs)
    return md



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午2:13
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : astparser.py
@Software: PyCharm
"""
import ast
import inspect

from .node import *
from ..docstring.parser import parse


class AstParser:
    def __init__(self, code: str, title: Optional[str] = None, style: str = "google", file_path: Optional[str] = None):
        """
        从一个文件的代码解析AST
        Args:
            code: 代码
            style: 注释风格
            title: 模块标题
            file_path: Python文件路径
        """
        self.style = style
        self.code = code
        self.tree = ast.parse(code)
        self.title = title
        """模块标题, 通常位于文件开头的单行注释, 会被解析为h1"""

        self.file_path = file_path

        self.description = parse(ast.get_docstring(self.tree), parser=self.style) if ast.get_docstring(self.tree) else None
        """模块描述, 通常位于文件开头的多行注释"""
        self.classes: list[ClassNode] = []
        self.functions: list[FunctionNode] = []
        self.variables: list[AssignNode] = []

        self.all_nodes: list[ClassNode | FunctionNode | AssignNode] = []

        self.parse()

    @staticmethod
    def clear_quotes(s: str) -> str:
        """
        去除类型注解中的引号
        Args:
            s:
        Returns:
        """
        return s.replace("'", "").replace('"', "")

    def get_line_content(self, lineno: int, ignore_index_out: bool = True) -> str:
        """获取代码行内容
        Args:
            lineno: 行号
            ignore_index_out: 是否忽略索引越界
        Returns:
            代码行内容
        """
        if ignore_index_out:
            if lineno < 1 or lineno > len(self.code.split("\n")):
                return ""
        return self.code.split("\n")[lineno - 1]

    @staticmethod
    def match_line_docs(linecontent: str) -> str:
        """匹配行内注释
        Args:
            linecontent: 行内容
        Returns:
            文档字符串
        """
        in_string = False
        string_char = ''
        for i, char in enumerate(linecontent):
            if char in ('"', "'"):
                if in_string:
                    if char == string_char:
                        in_string = False
                else:
                    in_string = True
                    string_char = char
            elif char == '#' and not in_string:
                return linecontent[i + 1:].strip()
        return ""

    def parse(self):
        for node in ast.walk(self.tree):
            if isinstance(node, ast.ClassDef):
                if not self._is_module_level_class(node):
                    continue

                class_node = ClassNode(
                    name=node.name,
                    docs=parse(ast.get_docstring(node), parser=self.style) if ast.get_docstring(node) else None,
                    inherits=[ast.unparse(base) for base in node.bases]
                )
                self.classes.append(class_node)
                self.all_nodes.append(class_node)

                # 继续遍历类内部的函数
                for sub_node in node.body:
                    if isinstance(sub_node, (ast.FunctionDef, ast.AsyncFunctionDef)):
                        class_node.methods.append(FunctionNode(
                            name=sub_node.name,
                            docs=parse(ast.get_docstring(sub_node), parser=self.style) if ast.get_docstring(sub_node) else None,
                            posonlyargs=[
                                    ArgNode(
                                        name=arg.arg,
                                        type=self.clear_quotes(ast.unparse(arg.annotation).strip()) if arg.annotation else TypeHint.NO_TYPEHINT,
                                    )
                                    for arg in sub_node.args.posonlyargs
                            ],
                            args=[
                                    ArgNode(
                                        name=arg.arg,
                                        type=self.clear_quotes(ast.unparse(arg.annotation).strip()) if arg.annotation else TypeHint.NO_TYPEHINT,
                                    )
                                    for arg in sub_node.args.args
                            ],
                            vararg=ArgNode(
                                name=sub_node.args.vararg.arg,
                                type=self.clear_quotes(
                                    ast.unparse(sub_node.args.vararg.annotation).strip()) if sub_node.args.vararg.annotation else TypeHint.NO_TYPEHINT
                            ) if sub_node.args.vararg else None,
                            kwonlyargs=[
                                    ArgNode(
                                        name=arg.arg,
                                        type=self.clear_quotes(ast.unparse(arg.annotation).strip()) if arg.annotation else TypeHint.NO_TYPEHINT,
                                    )
                                    for arg in sub_node.args.kwonlyargs
                            ],
                            kwarg=ArgNode(
                                name=sub_node.args.kwarg.arg,
                                type=self.clear_quotes(
                                    ast.unparse(sub_node.args.kwarg.annotation).strip()) if sub_node.args.kwarg.annotation else TypeHint.NO_TYPEHINT
                            ) if sub_node.args.kwarg else None,
                            kw_defaults=[
                                    ConstantNode(
                                        value=ast.unparse(default).strip() if default else TypeHint.NO_DEFAULT
                                    )
                                    for default in sub_node.args.kw_defaults
                            ],
                            defaults=[
                                    ConstantNode(
                                        value=ast.unparse(default).strip() if default else TypeHint.NO_DEFAULT
                                    )
                                    for default in sub_node.args.defaults
                            ],
                            return_=self.clear_quotes(ast.unparse(sub_node.returns).strip()) if sub_node.returns else TypeHint.NO_RETURN,
                            decorators=[ast.unparse(decorator).strip() for decorator in sub_node.decorator_list],
                            is_async=isinstance(sub_node, ast.AsyncFunctionDef),
                            src=ast.unparse(sub_node).strip(),
                            is_classmethod=True,
                            lineno=sub_node.lineno,
                            module_file_path=self.file_path
                        ))
                    elif isinstance(sub_node, (ast.Assign, ast.AnnAssign)):
                        if isinstance(sub_node, ast.Assign):
                            class_node.attrs.append(AttrNode(
                                name=sub_node.targets[0].id,  # type: ignore
                                type=TypeHint.NO_TYPEHINT,
                                value=ast.unparse(sub_node.value).strip()
                            ))
                        elif isinstance(sub_node, ast.AnnAssign):
                            class_node.attrs.append(AttrNode(
                                name=sub_node.target.id,
                                type=ast.unparse(sub_node.annotation).strip(),
                                value=ast.unparse(sub_node.value).strip() if sub_node.value else TypeHint.NO_DEFAULT
                            ))
                        else:
                            raise ValueError(f"Unsupported node type: {type(sub_node)}")

            elif isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
                # 仅打印模块级别的函数
                if not self._is_module_level_function(node):
                    continue
                function_node = FunctionNode(
                    name=node.name,
                    docs=parse(ast.get_docstring(node), parser=self.style) if ast.get_docstring(node) else None,
                    posonlyargs=[
                            ArgNode(
                                name=arg.arg,
                                type=self.clear_quotes(ast.unparse(arg.annotation).strip()) if arg.annotation else TypeHint.NO_TYPEHINT,
                            )
                            for arg in node.args.posonlyargs
                    ],
                    args=[
                            ArgNode(
                                name=arg.arg,
                                type=self.clear_quotes(ast.unparse(arg.annotation).strip()) if arg.annotation else TypeHint.NO_TYPEHINT,
                            )
                            for arg in node.args.args
                    ],
                    vararg=ArgNode(
                        name=node.args.vararg.arg,
                        type=self.clear_quotes(ast.unparse(node.args.vararg.annotation).strip()) if node.args.vararg.annotation else TypeHint.NO_TYPEHINT
                    ) if node.args.vararg else None,
                    kwonlyargs=[
                            ArgNode(
                                name=arg.arg,
                                type=self.clear_quotes(ast.unparse(arg.annotation).strip()) if arg.annotation else TypeHint.NO_TYPEHINT,
                            )
                            for arg in node.args.kwonlyargs
                    ],
                    kwarg=ArgNode(
                        name=node.args.kwarg.arg,
                        type=self.clear_quotes(ast.unparse(node.args.kwarg.annotation).strip()) if node.args.kwarg.annotation else TypeHint.NO_TYPEHINT
                    ) if node.args.kwarg else None,
                    kw_defaults=[
                            ConstantNode(
                                value=ast.unparse(default).strip() if default else TypeHint.NO_DEFAULT
                            )
                            for default in node.args.kw_defaults
                    ],
                    defaults=[
                            ConstantNode(
                                value=ast.unparse(default).strip() if default else TypeHint.NO_DEFAULT
                            )
                            for default in node.args.defaults
                    ],
                    return_=self.clear_quotes(ast.unparse(node.returns).strip()) if node.returns else TypeHint.NO_RETURN,
                    decorators=[ast.unparse(decorator).strip() for decorator in node.decorator_list],
                    is_async=isinstance(node, ast.AsyncFunctionDef),
                    src=ast.unparse(node).strip(),
                    lineno=node.lineno,
                    module_file_path=self.file_path
                )
                self.functions.append(function_node)
                self.all_nodes.append(function_node)

            elif isinstance(node, (ast.Assign, ast.AnnAssign)):
                if not self._is_module_level_variable2(node):
                    continue
                else:
                    pass
                lineno = node.lineno
                prev_line = self.get_line_content(lineno - 1).strip()
                curr_line = self.get_line_content(lineno).strip()
                next_line = self.get_line_content(lineno + 1).strip()

                # 获取文档字符串，优先检测下行"""
                if next_line.startswith('"""'):
                    docs = next_line[3:-3]
                elif prev_line.startswith('"""'):
                    docs = prev_line[3:-3]
                else:
                    curr_docs = self.match_line_docs(curr_line)
                    if curr_docs:
                        docs = curr_docs
                    else:
                        docs = None

                if isinstance(node, ast.Assign):
                    for target in node.targets:
                        if isinstance(target, ast.Name):
                            ass_node = AssignNode(
                                name=target.id,
                                value=ast.unparse(node.value).strip(),
                                type=ast.unparse(node.annotation).strip() if isinstance(node, ast.AnnAssign) else TypeHint.NO_TYPEHINT,
                                docs=docs
                            )
                            self.variables.append(ass_node)
                            self.all_nodes.append(ass_node)
                if isinstance(node, ast.AnnAssign):
                    annass_node = AssignNode(
                        name=node.target.id,
                        value=ast.unparse(node.value).strip() if node.value else TypeHint.NO_DEFAULT,
                        type=ast.unparse(node.annotation).strip(),
                        docs=docs
                    )
                    self.variables.append(annass_node)
                    self.all_nodes.append(annass_node)

    def _is_module_level_function(self, node: ast.FunctionDef | ast.AsyncFunctionDef):
        for parent in ast.walk(self.tree):
            if isinstance(parent, (ast.ClassDef, ast.FunctionDef, ast.AsyncFunctionDef)):
                if node in parent.body:
                    return False
        return True

    def _is_module_level_class(self, node: ast.ClassDef):
        for parent in ast.walk(self.tree):
            if isinstance(parent, ast.ClassDef):
                if node in parent.body:
                    return False
        return True

    def _is_module_level_variable(self, node: ast.Assign | ast.AnnAssign):
        """在类方法或函数内部的变量不会被记录"""

        # for parent in ast.walk(self.tree):
        #     if isinstance(parent, (ast.ClassDef, ast.FunctionDef, ast.AsyncFunctionDef)):
        #         if node in parent.body:
        #             return False
        #         else:
        #             for sub_node in parent.body:
        #                 if isinstance(sub_node, (ast.FunctionDef, ast.AsyncFunctionDef)):
        #                     if node in sub_node.body:
        #                         return False
        # return True
        # 递归检查
        def _check(_node, _parent):
            if isinstance(_parent, (ast.ClassDef, ast.FunctionDef, ast.AsyncFunctionDef)):
                if _node in _parent.body:
                    return False
                else:
                    for sub_node in _parent.body:
                        if isinstance(sub_node, (ast.FunctionDef, ast.AsyncFunctionDef)):
                            return _check(_node, sub_node)
            return True

        for parent in ast.walk(self.tree):
            if not _check(node, parent):
                return False
        return True

    def _is_module_level_variable2(self, node: ast.Assign | ast.AnnAssign) -> bool:
        """
        检查变量是否在模块级别定义。
        """
        for parent in ast.walk(self.tree):
            if isinstance(parent, (ast.ClassDef, ast.FunctionDef, ast.AsyncFunctionDef)):
                if node in parent.body:
                    return False
        return True

    def __str__(self):
        s = ""
        for cls in self.classes:
            s += f"class {cls.name}:\n"
        for func in self.functions:
            s += f"def {func.name}:\n"
        for var in self.variables:
            s += f"{var.name} = {var.value}\n"
        return s



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午1:46
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : docstring.py
@Software: PyCharm
"""
from typing import Optional

from pydantic import BaseModel, Field

from litedoc.i18n import get_text


class Attr(BaseModel):
    name: str
    type: str = ""
    desc: str = ""


class Args(BaseModel):
    name: str
    type: str = ""
    desc: str = ""


class Return(BaseModel):
    desc: str = ""


class Exception_(BaseModel):
    name: str
    desc: str = ""


class Raise(BaseModel):
    exceptions: list[Exception_] = []


class Example(BaseModel):
    desc: str = ""
    input: str = ""
    output: str = ""


class Docstring(BaseModel):
    raw: str = ""
    desc: str = ""
    args: list[Args] = []
    attrs: list[Attr] = []
    return_: Optional[Return] = None
    raise_: list[Exception_] = []
    example: Optional[str] = None

    def add_desc(self, desc: str):
        if self.desc == "":
            self.desc = desc
        else:
            self.desc += "\n" + desc

    def add_arg(self, name: str, type_: str = "", desc: str = ""):
        self.args.append(Args(name=name, type=type_, desc=desc))

    def add_attrs(self, name: str, type_: str = "", desc: str = ""):
        self.attrs.append(Attr(name=name, type=type_, desc=desc))

    def add_return(self, desc: str = ""):
        self.return_ = Return(desc=desc)

    def add_raise(self, name: str, desc: str = ""):
        self.raise_.append(Exception_(name=name, desc=desc))

    def add_example(self, desc: str = ""):
        if self.example is None:
            self.example = desc
        else:
            self.example += "\n" + desc

    def reduction(self, style: str = "google") -> str:
        """
        通过解析结果还原docstring
        Args:
            style: docstring风格
        Returns:

        """
        ret = ""
        if style == "google":
            ret += self.desc + "\n"
            if self.args:
                ret += "Args:\n"
                for arg in self.args:
                    ret += f"    {arg.name}: {arg.type}\n        {arg.desc}\n"
            if self.attrs:
                ret += "Attributes:\n"
                for attr in self.attrs:
                    ret += f"    {attr.name}: {attr.type}\n        {attr.desc}\n"
            if self.return_:
                ret += "Returns:\n"
                ret += f"    {self.return_.desc}\n"

            if self.raise_:
                ret += "Raises:\n"
                for exception in self.raise_:
                    ret += f"    {exception.name}\n        {exception.desc}\n"

            if self.example:
                ret += "Examples:\n"
                ret += f"    {self.example}\n"
        return ret

    def markdown(self, lang: str, indent: int = 4) -> str:
        """
        生成markdown文档
        Args:
            lang:
            indent:

        Returns:

        """
        PREFIX = "" * indent
        ret = ""
        # ret += self.desc + "\n\n"
        # print(self.reduction())
        # print(self.desc, self.return_)
        # 单数属性
        if self.desc:
            ret += PREFIX + f"\n**{get_text(lang, 'desc')}**: {self.desc}\n"

        # 复数属性
        if self.args:
            ret += PREFIX + f"\n**{get_text(lang, 'docstring.args')}**:\n"
            for arg in self.args:
                ret += PREFIX + f"> - {arg.name}: {arg.type}  {arg.desc}\n"
        if self.attrs:
            ret += PREFIX + f"\n**{get_text(lang, 'docstring.attrs')}**:\n"
            for attr in self.attrs:
                ret += PREFIX + f"> - {attr.name}: {attr.type}  {attr.desc}\n"

        # 单数属性
        if self.return_ is not None:
            ret += PREFIX + f"\n**{get_text(lang, 'docstring.return')}**: {self.return_.desc}\n"
        # 复数属性
        if self.example:
            ret += PREFIX + f"\n**{get_text(lang, 'docstring.example')}**:\n"
            ret += self.example + "\n"
        if self.raise_:
            ret += PREFIX + f"\n**{get_text(lang, 'docstring.raises')}**:\n"
            for exception in self.raise_:
                ret += PREFIX + f"> - {exception.name}  {exception.desc}\n"

        return ret

    def __str__(self):
        return self.desc



# -*- coding: utf-8 -*-
"""
几何模型点集
"""
import numpy as np
from mbcp.mp_math.point import Point3
from mbcp.mp_math.utils import clamp  # type: ignore


class GeometricModels:
    @staticmethod
    def sphere(radius: float, density: float):
        """
        生成球体上的点集。
        Args:
            radius:
            density:
        Returns:
            List[Point3]: 球体上的点集。
        """
        area = 4 * np.pi * radius ** 2  # 球表面积
        num = int(area * density)  # 粒子数
        phi_list = np.arccos([clamp(-1 + ((2.0 * _ - 1.0) / num), -1, 1) for _ in range(num)])
        theta_list = np.sqrt(num * np.pi) * phi_list
        x_array = radius * np.sin(phi_list) * np.cos(theta_list)
        y_array = radius * np.sin(phi_list) * np.sin(theta_list)
        z_array = radius * np.cos(phi_list)
        return [Point3(x_array[i], y_array[i], z_array[i]) for i in range(num)]



# -*- coding: utf-8 -*-
"""
Internationalization module.
"""
from typing import Optional, TypeAlias

NestedDict: TypeAlias = dict[str, 'str | NestedDict']

"""@litedoc-hide"""
i18n_dict: dict[str, NestedDict] = {  # @litedoc-hide
        "en"     : {
                "docstring": {
                        "args"     : "Arguments",
                        "return"   : "Return",
                        "attribute": "Attribute",
                        "raises"   : "Raises",
                        "example"  : "Examples",
                        "yields"   : "Yields",
                },
                "src"      : "Source code",
                "desc"     : "Description",
                "type"     : "Type",
                "or"       : "or",
                "view_on_github": "View on GitHub",
                "default_value": "Default",
        },
        "zh-Hans": {
                "docstring": {
                        "args"     : "参数",
                        "return"   : "返回",
                        "attribute": "属性",
                        "raises"   : "引发",
                        "example"  : "示例",
                        "yields"   : "产出",
                },
                "src"      : "源代码",
                "desc"     : "说明",
                "type"     : "类型",
                "or"       : "或",
                "view_on_github": "在GitHub上查看",
                "default_value": "默认值",
        },
        "zh-Hant": {
                "docstring": {
                        "args"     : "變數説明",
                        "return"   : "返回",
                        "attribute": "屬性",
                        "raises"   : "抛出",
                        "example"  : "範例",
                        "yields"   : "產出",
                },
                "src"      : "源碼",
                "desc"     : "説明",
                "type"     : "類型",
                "or"       : "或",
                "view_on_github": "於GitHub上查看",
                "default_value": "默認值",
        },
        "ja"     : {
                "docstring": {
                        "args"     : "引数",
                        "return"   : "戻り値",
                        "attribute": "属性",
                        "raises"   : "例外",
                        "example"  : "例",
                        "yields"   : "生成",
                },
                "src"      : "ソースコード",
                "desc"     : "説明",
                "type"     : "タイプ",
                "or"       : "または",
                "view_on_github": "GitHubで表示",
                "default_value": "デフォルト",
        },
}  # @litedoc-hide


def flat_i18n_dict(data: dict[str, NestedDict]) -> dict[str, dict[str, str]]:
    """
    Flatten i18n_dict.
    Args:
        data: i18n_dict
    Returns:
        dict: flatten dict
    """
    ret: dict[str, dict[str, str]] = {}

    def _flat(_lang_data: NestedDict) -> dict[str, str]:
        res = {}
        for k, v in _lang_data.items():
            if isinstance(v, dict):
                for kk, vv in _flat(v).items():
                    res[f"{k}.{kk}"] = vv
            else:
                res[k] = v
        return res

    for lang, lang_data in data.items():
        ret[lang] = _flat(lang_data)

    return ret


i18n_flat_dict = flat_i18n_dict(i18n_dict)


def get_text(lang: str, key: str, default: Optional[str] = None, fallback: Optional[str] = "en") -> str:
    """
    Get text from i18n_dict.
    Args:
        lang: language name
        key: text key
        default: default text, if None return fallback language or key
        fallback: fallback language, priority is higher than default
    Returns:
        str: text
    """
    if lang in i18n_flat_dict:
        if key in i18n_flat_dict[lang]:
            return i18n_flat_dict[lang][key]

    if fallback is not None:
        return i18n_flat_dict.get(fallback, {}).get(key, default or key)
    else:
        return default or key



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午2:30
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : test_parser.py
@Software: PyCharm
"""
import os

from liteyuki_autodoc.syntax.astparser import AstParser


class TestParser:

    def test_one_file(self):
        file = "test_modules/tree.py"
        text = open(file, "r", encoding="utf-8").read()
        parser = AstParser(text)
        print(parser)



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/25 下午9:45
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : const.py
@Software: PyCharm
"""
import math


PI = math.pi
E = math.e
GOLDEN_RATIO = (1 + math.sqrt(5)) / 2
GAMMA = 0.57721566490153286060651209008240243104215933593992
EPSILON = 0.0001
"""ε"""
APPROX = 0.001
"""约等于误差"""



from abc import abstractmethod


def custom_decorator(func):
    def wrapper(*args, **kwargs):
        print("decorator")
        return func(*args, **kwargs)
    return wrapper

class B:
    ...


class A(B):
    """
    A class
    """

    def __init__(self):
        """
        A class constructor
        """
        self.a = 1
        self.a_prop: str = ""
        """类属性注解测试"""

    def a_method(self, arg1: int, arg2: str = ""):
        """
        A class method
        """
        print(arg1, arg2, self.a)

    @property
    def a_p(self) -> int:
        return self.a

    @classmethod
    def c_method(cls):
        print(cls)

    @staticmethod
    def s_method():
        print("static")

    @abstractmethod
    def abs_method(self):
        pass

    @custom_decorator
    def custom_method(self):
        print("custom")

def b_method(a, b=2, c=3):
    print(a)

async def async_func():
    print("async")

# 单行注释
CONST = 3
"""变量注解测试"""



# -*- coding: utf-8 -*-
"""
平面模块
"""
import math
from typing import TYPE_CHECKING, overload

import numpy as np

from .const import APPROX
from .vector import Vector3, zero_vector3
from .line import Line3
from .point import Point3
from .utils import approx, sign

if TYPE_CHECKING:
    from .angle import AnyAngle


class Plane3:
    def __init__(self, a: float, b: float, c: float, d: float):
        """
        平面方程：ax + by + cz + d = 0
        Args:
            a: x系数
            b: y系数
            c: z系数
            d: 常数项
        """
        self.a = a
        self.b = b
        self.c = c
        self.d = d

    def approx(self, other: 'Plane3') -> bool:
        """
        判断两个平面是否近似相等。
        Args:
            other: 另一个平面
        Returns:
            是否近似相等
        """
        if self.a != 0:
            k = other.a / self.a
            return approx(other.b, self.b * k) and approx(other.c, self.c * k) and approx(other.d, self.d * k)
        elif self.b != 0:
            k = other.b / self.b
            return approx(other.a, self.a * k) and approx(other.c, self.c * k) and approx(other.d, self.d * k)
        elif self.c != 0:
            k = other.c / self.c
            return approx(other.a, self.a * k) and approx(other.b, self.b * k) and approx(other.d, self.d * k)
        else:
            return False

    def cal_angle(self, other: 'Line3 | Plane3') -> 'AnyAngle':
        """
        计算平面与平面之间的夹角。
        Args:
            other: 另一个平面
        Returns:
            夹角弧度
        Raises:
            TypeError: 不支持的类型
        """
        if isinstance(other, Line3):
            return self.normal.cal_angle(other.direction).complementary
        elif isinstance(other, Plane3):
            return AnyAngle(math.acos(self.normal @ other.normal / (self.normal.length * other.normal.length)), is_radian=True)
        else:
            raise TypeError(f"Unsupported type: {type(other)}")

    def cal_distance(self, other: 'Plane3 | Point3') -> float:
        """
        计算平面与平面或点之间的距离。
        Args:
            other: 另一个平面或点
        Returns:
            距离
        Raises:
            TypeError: 不支持的类型
        """
        if isinstance(other, Plane3):
            return 0
        elif isinstance(other, Point3):
            return abs(self.a * other.x + self.b * other.y + self.c * other.z + self.d) / (self.a ** 2 + self.b ** 2 + self.c ** 2) ** 0.5
        else:
            raise TypeError(f"Unsupported type: {type(other)}")

    def cal_intersection_line3(self, other: 'Plane3') -> 'Line3':
        """
        计算两平面的交线。
        Args:
            other: 另一个平面
        Returns:
            两平面的交线
        Raises:
        """
        if self.normal.is_parallel(other.normal):
            raise ValueError("Planes are parallel and have no intersection.")
        direction = self.normal.cross(other.normal)  # 法向量叉乘得到方向向量
        # 寻找直线上的一点，依次假设x=0, y=0, z=0，找到合适的点
        x, y, z = 0, 0, 0
        # 依次判断条件假设x=0, y=0, z=0，找到合适的点
        # 先假设其中一个系数不为0，则令此坐标为0，构建增广矩阵，解出另外两个坐标

        if self.a != 0 and other.a != 0:
            A = np.array([[self.b, self.c], [other.b, other.c]])
            B = np.array([-self.d, -other.d])
            y, z = np.linalg.solve(A, B)
        elif self.b != 0 and other.b != 0:
            A = np.array([[self.a, self.c], [other.a, other.c]])
            B = np.array([-self.d, -other.d])
            x, z = np.linalg.solve(A, B)
        elif self.c != 0 and other.c != 0:
            A = np.array([[self.a, self.b], [other.a, other.b]])
            B = np.array([-self.d, -other.d])
            x, y = np.linalg.solve(A, B)

        return Line3(Point3(x, y, z), direction)

    def cal_intersection_point3(self, other: 'Line3') -> 'Point3':
        """
        计算平面与直线的交点。
        Args:
            other: 不与平面平行或在平面上的直线
        Returns:
            交点
        Raises:
            ValueError: 平面与直线平行或重合
        """
        # 若平面的法向量与直线方向向量垂直，则直线与平面平行或重合
        if self.normal @ other.direction == 0:
            raise ValueError("The plane and the line are parallel or coincident.")
        # 获取直线的参数方程
        # 代入平面方程，解出t
        x, y, z = other.get_parametric_equations()
        t = (-(self.a * other.point.x + self.b * other.point.y + self.c * other.point.z + self.d) /
             (self.a * other.direction.x + self.b * other.direction.y + self.c * other.direction.z))
        return Point3(x(t), y(t), z(t))

    def cal_parallel_plane3(self, point: 'Point3') -> 'Plane3':
        """
        计算平行于该平面且过指定点的平面。
        Args:
            point: 指定点
        Returns:
            所求平面
        """
        return Plane3.from_point_and_normal(point, self.normal)

    def is_parallel(self, other: 'Plane3') -> bool:
        """
        判断两个平面是否平行。
        Args:
            other: 另一个平面
        Returns:
            是否平行
        """
        return self.normal.is_parallel(other.normal)

    @property
    def normal(self) -> 'Vector3':
        """
        平面的法向量。
        Returns:
            法向量
        """
        return Vector3(self.a, self.b, self.c)

    @classmethod
    def from_point_and_normal(cls, point: 'Point3', normal: 'Vector3') -> 'Plane3':
        """
        工厂函数 由点和法向量构造平面(点法式构造)。
        Args:
            point: 平面上的一点
            normal: 法向量
        Returns:
            平面
        """
        a, b, c = normal.x, normal.y, normal.z
        d = -a * point.x - b * point.y - c * point.z  # d = -ax - by - cz
        return cls(a, b, c, d)

    @classmethod
    def from_three_points(cls, p1: 'Point3', p2: 'Point3', p3: 'Point3') -> 'Plane3':
        """
        工厂函数 由三点构造平面。
        Args:
            p1: 点1
            p2: 点2
            p3: 点3
        Returns:
            平面
        """
        # 两个向量
        v1 = p2 - p1
        v2 = p3 - p1
        # 法向量
        normal = v1.cross(v2)
        return cls.from_point_and_normal(p1, normal)

    @classmethod
    def from_two_lines(cls, l1: 'Line3', l2: 'Line3') -> 'Plane3':
        """
        工厂函数 由两直线构造平面。
        Args:
            l1: 直线1
            l2: 直线2
        Returns:
            平面
        """
        v1 = l1.direction
        v2 = l2.point - l1.point
        if v2 == zero_vector3:
            v2 = l2.get_point(1) - l1.point
        return cls.from_point_and_normal(l1.point, v1.cross(v2))

    @classmethod
    def from_point_and_line(cls, point: 'Point3', line: 'Line3') -> 'Plane3':
        """
        工厂函数 由点和直线构造平面。
        Args:
            point: 面上一点
            line: 面上直线，不包含点
        Returns:
            平面
        """
        return cls.from_point_and_normal(point, line.direction)

    def __repr__(self):
        return f"Plane3({self.a}, {self.b}, {self.c}, {self.d})"

    def __str__(self):
        s = "Plane3: "
        if self.a != 0:
            s += f"{sign(self.a, only_neg=True)}{abs(self.a)}x"
        if self.b != 0:
            s += f" {sign(self.b)} {abs(self.b)}y"
        if self.c != 0:
            s += f" {sign(self.c)} {abs(self.c)}z"
        if self.d != 0:
            s += f" {sign(self.d)} {abs(self.d)}"
        return s + " = 0"

    @overload
    def __and__(self, other: 'Line3') -> 'Point3 | None':
        ...

    @overload
    def __and__(self, other: 'Plane3') -> 'Line3 | None':
        ...

    def __and__(self, other):
        """
        取两平面的交集（人话：交线）
        Args:
            other:
        Returns:
            不平行平面的交线，平面平行返回None
        """
        if isinstance(other, Plane3):
            if self.normal.is_parallel(other.normal):
                return None
            return self.cal_intersection_line3(other)
        elif isinstance(other, Line3):
            if self.normal @ other.direction == 0:
                return None
            return self.cal_intersection_point3(other)
        else:
            raise TypeError(f"unsupported operand type(s) for &: 'Plane3' and '{type(other)}'")

    def __eq__(self, other) -> bool:
        return self.approx(other)

    def __rand__(self, other: 'Line3') -> 'Point3':
        return self.cal_intersection_point3(other)





import math
from typing import overload, TYPE_CHECKING

import numpy as np

from .const import APPROX
from .mp_math_typing import RealNumber
from .point import Point3
from .angle import AnyAngle
from .utils import approx

# if TYPE_CHECKING:
#


class Vector3:
    def __init__(self, x: float, y: float, z: float):
        """
        3维向量
        Args:
            x: x轴分量
            y: y轴分量
            z: z轴分量
        """
        self.x = x
        self.y = y
        self.z = z

    def approx(self, other: 'Vector3', epsilon: float = APPROX) -> bool:
        """
        判断两个向量是否近似相等。
        Args:
            other:
            epsilon:

        Returns:
            是否近似相等
        """
        return all([abs(self.x - other.x) < epsilon, abs(self.y - other.y) < epsilon, abs(self.z - other.z) < epsilon])

    def cal_angle(self, other: 'Vector3') -> 'AnyAngle':
        """
        计算两个向量之间的夹角。
        Args:
            other: 另一个向量
        Returns:
            夹角
        """
        return AnyAngle(math.acos(self @ other / (self.length * other.length)), is_radian=True)

    def cross(self, other: 'Vector3') -> 'Vector3':
        """
        向量积 叉乘：v1 cross v2 -> v3

        叉乘为0，则两向量平行。
        其余结果的模为平行四边形的面积。

        返回如下行列式的结果：

        ``i  j  k``

        ``x1 y1 z1``

        ``x2 y2 z2``

        Args:
            other:
        Returns:
            行列式的结果
        """
        return Vector3(self.y * other.z - self.z * other.y,
                       self.z * other.x - self.x * other.z,
                       self.x * other.y - self.y * other.x)

    def is_approx_parallel(self, other: 'Vector3', epsilon: float = APPROX) -> bool:
        """
        判断两个向量是否近似平行。
        Args:
            other: 另一个向量
            epsilon: 允许的误差
        Returns:
            是否近似平行
        """
        return self.cross(other).length < epsilon

    def is_parallel(self, other: 'Vector3') -> bool:
        """
        判断两个向量是否平行。
        Args:
            other: 另一个向量
        Returns:
            是否平行
        """
        return self.cross(other).approx(zero_vector3)

    def normalize(self):
        """
        将向量归一化。

        自体归一化，不返回值。
        """
        length = self.length
        self.x /= length
        self.y /= length
        self.z /= length

    @property
    def np_array(self) -> 'np.ndarray':
        """
        返回numpy数组
        Returns:
        """

        return np.array([self.x, self.y, self.z])

    @property
    def length(self) -> float:
        """
        向量的模。
        Returns:
            模
        """
        return math.sqrt(self.x ** 2 + self.y ** 2 + self.z ** 2)

    @property
    def unit(self) -> 'Vector3':
        """
        获取该向量的单位向量。
        Returns:
            单位向量
        """
        return self / self.length

    def __abs__(self):
        return self.length

    @overload
    def __add__(self, other: 'Vector3') -> 'Vector3':
        ...

    @overload
    def __add__(self, other: 'Point3') -> 'Point3':
        ...

    def __add__(self, other):
        """
        V + P -> P\n
        V + V -> V
        Args:
            other:
        Returns:

        """
        if isinstance(other, Vector3):
            return Vector3(self.x + other.x, self.y + other.y, self.z + other.z)
        elif isinstance(other, Point3):
            return Point3(self.x + other.x, self.y + other.y, self.z + other.z)
        else:
            raise TypeError(f"unsupported operand type(s) for +: 'Vector3' and '{type(other)}'")

    def __eq__(self, other):
        """
        判断两个向量是否相等。
        Args:
            other:
        Returns:
            是否相等
        """
        return approx(self.x, other.x) and approx(self.y, other.y) and approx(self.z, other.z)

    def __radd__(self, other: 'Point3') -> 'Point3':
        """
        P + V -> P\n
        别去点那边实现了。
        :param other:
        :return:
        """
        return Point3(self.x + other.x, self.y + other.y, self.z + other.z)

    @overload
    def __sub__(self, other: 'Vector3') -> 'Vector3':
        ...

    @overload
    def __sub__(self, other: 'Point3') -> "Point3":
        ...

    def __sub__(self, other):
        """
        V - P -> P\n
        V - V -> V
        Args:
            other:
        Returns:
        """
        if isinstance(other, Vector3):
            return Vector3(self.x - other.x, self.y - other.y, self.z - other.z)
        elif isinstance(other, Point3):
            return Point3(self.x - other.x, self.y - other.y, self.z - other.z)
        else:
            raise TypeError(f"unsupported operand type(s) for -: \"Vector3\" and \"{type(other)}\"")

    def __rsub__(self, other: 'Point3'):
        """
        P - V -> P
        Args:
            other:
        Returns:

        """

        if isinstance(other, Point3):
            return Point3(other.x - self.x, other.y - self.y, other.z - self.z)
        else:
            raise TypeError(f"unsupported operand type(s) for -: '{type(other)}' and 'Vector3'")

    @overload
    def __mul__(self, other: 'Vector3') -> 'Vector3':
        ...

    @overload
    def __mul__(self, other: RealNumber) -> 'Vector3':
        ...

    def __mul__(self, other: 'int | float | Vector3') -> 'Vector3':
        """
        数组运算 非点乘。点乘使用@，叉乘使用cross。
        Args:
            other:

        Returns:
        """

        if isinstance(other, Vector3):
            return Vector3(self.x * other.x, self.y * other.y, self.z * other.z)
        elif isinstance(other, (float, int)):
            return Vector3(self.x * other, self.y * other, self.z * other)
        else:
            raise TypeError(f"unsupported operand type(s) for *: 'Vector3' and '{type(other)}'")

    def __rmul__(self, other: 'RealNumber') -> 'Vector3':
        return self.__mul__(other)

    def __matmul__(self, other: 'Vector3') -> 'RealNumber':
        """
        点乘。
        Args:
            other:
        Returns:
        """
        return self.x * other.x + self.y * other.y + self.z * other.z

    def __truediv__(self, other: RealNumber) -> 'Vector3':
        return Vector3(self.x / other, self.y / other, self.z / other)

    def __neg__(self):
        return Vector3(-self.x, -self.y, -self.z)

    def __repr__(self):
        return f"Vector3({self.x}, {self.y}, {self.z})"

    def __str__(self):
        return f"Vector3({self.x}, {self.y}, {self.z})"


zero_vector3: Vector3 = Vector3(0, 0, 0)
"""零向量"""
x_axis: Vector3 = Vector3(1, 0, 0)
"""x轴单位向量"""
y_axis: Vector3 = Vector3(0, 1, 0)
"""y轴单位向量"""
z_axis: Vector3 = Vector3(0, 0, 1)
"""z轴单位向量"""



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/28 下午2:14
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : node.py
@Software: PyCharm
"""
from typing import Literal, Optional
from enum import Enum

from pydantic import BaseModel, Field

from litedoc.docstring.docstring import Docstring
from litedoc.i18n import get_text


class TypeHint:
    NO_TYPEHINT = "NO_TYPE_HINT"
    NO_DEFAULT = "NO_DEFAULT"
    NO_RETURN = "NO_RETURN"


class AssignNode(BaseModel):
    """
    AssignNode is a pydantic model that represents an assignment.
    Attributes:
        name: str
            The name of the assignment.
        type: str = ""
            The type of the assignment.
        value: str
            The value of the assignment.
    """
    name: str
    type: str = ""
    value: str
    docs: Optional[str] = ""

    def markdown(self, lang: str, **kwargs) -> str:
        """
        Args:
            lang: str
                The language of the
        Returns:
            markdown style document
        """
        vd = kwargs.get("vd", "var")
        md = ""
        md += f"### {vd} `{self.name}`\n\n"
        if self.docs is not None:
            md += f"- **{get_text(lang, 'desc')}**: {self.docs}\n\n"
        if self.type != TypeHint.NO_TYPEHINT:
            md += f"- **{get_text(lang, 'type')}**: `{self.type}`\n\n"
        md += f"- **{get_text(lang, 'default_value')}**: `{self.value}`\n\n"


        return md


class ArgNode(BaseModel):
    """
    ArgNode is a pydantic model that represents an argument.
    Attributes:
        name: str
            The name of the argument.
        type: str = ""
            The type of the argument.
        default: str = ""
            The default value of the argument.
    """
    name: str
    type: str = TypeHint.NO_TYPEHINT


class AttrNode(BaseModel):
    """
    AttrNode is a pydantic model that represents an attribute.
    Attributes:
        name: str
            The name of the attribute.
        type: str = ""
            The type of the attribute.
        value: str = ""
            The value of the attribute
    """
    name: str
    type: str = ""
    value: str = ""


class ImportNode(BaseModel):
    """
    ImportNode is a pydantic model that represents an import statement.
    Attributes:
        name: str
            The name of the import statement.
        as_: str = ""
            The alias of the import
    """
    name: str
    as_: str = ""


class ConstantNode(BaseModel):
    """
    ConstantNode is a pydantic model that represents a constant.
    Attributes:
        value: str
            The value of the constant.
    """
    value: str


class FunctionNode(BaseModel):
    """
    FunctionNode is a pydantic model that represents a function.
    Attributes:
        name: str
            The name of the function.
        docs: str = ""
            The docstring of the function.
        args: list[ArgNode] = []
            The arguments of the function.
        return_: ReturnNode = None
            The return value of the function.
        decorators: list[str] = []
            The decorators of the function.
        is_async: bool = False
            Whether the function is asynchronous.
    """
    name: str
    docs: Optional[Docstring] = None

    posonlyargs: list[ArgNode] = []
    args: list[ArgNode] = []
    vararg: Optional[ArgNode] = None
    kwonlyargs: list[ArgNode] = []
    kwarg: Optional[ArgNode] = None
    kw_defaults: list[ConstantNode] = []
    defaults: list[ConstantNode] = []

    lineno: int = 0
    module_file_path: str = ""  # 去头路径，不包含模块顶级文件夹

    return_: str = TypeHint.NO_RETURN
    decorators: list[str] = []
    src: str
    is_async: bool = False
    is_classmethod: bool = False

    magic_methods: dict[str, str] = {
            "__add__"     : "+",
            "__radd__"    : "+",
            "__sub__"     : "-",
            "__rsub__"    : "-",
            "__mul__"     : "*",
            "__rmul__"    : "*",
            "__matmul__"  : "@",
            "__rmatmul__" : "@",
            "__mod__"     : "%",
            "__truediv__" : "/",
            "__rtruediv__": "/",
            "__neg__"     : "-",
    }  # 魔术方法, 例如运算符

    def is_private(self):
        """
        Check if the function or method is private.
        Returns:
            bool: True if the function or method is private, False otherwise.
        """
        return self.name.startswith("_")

    def is_builtin(self):
        """
        Check if the function or method is a builtin function or method.
        Returns:
            bool: True if the function or method is a builtin function or method, False otherwise.
        """
        return self.name.startswith("__") and self.name.endswith("__")

    def markdown(self, lang: str, indent: int = 0, **kwargs) -> str:
        """
        Args:
            indent: int
                The number of spaces to indent the markdown.
            lang: str
                The language of the
            **kwargs: more parameters
        Returns:
            markdown style document
        """
        self.complete_default_args()
        PREFIX = "" * indent
        # if is_classmethod:
        #     PREFIX = "- #"
        func_type = kwargs.get("fd", "func") if not self.is_classmethod else kwargs.get("md", "method")
        h_level = 3 if not self.is_classmethod else 4
        h = "#" * h_level
        """标题等级"""

        md = ""
        # 装饰器部分
        if len(self.decorators) > 0:
            for decorator in self.decorators:
                md += PREFIX + f"`@{decorator}`\n"

        if self.is_async:
            md += PREFIX + f"{h} ***async {func_type}*** "
        else:
            md += PREFIX + f"{h} ***{func_type}*** "

        # code start
        # 配对位置参数和位置参数默认值，无默认值用TypeHint.NO_DEFAULT
        args: list[str] = []  # 可直接", ".join(args)得到位置参数部分
        arg_i = 0

        if len(self.posonlyargs) > 0:
            for arg in self.posonlyargs:
                arg_text = f"{arg.name}"
                if arg.type != TypeHint.NO_TYPEHINT:
                    arg_text += f": {arg.type}"
                arg_default = self.defaults[arg_i].value
                if arg_default != TypeHint.NO_DEFAULT:
                    arg_text += f" = {arg_default}"
                args.append(arg_text)
                arg_i += 1
            # 加位置参数分割符  /
            args.append("/")

        for arg in self.args:
            arg_text = f"{arg.name}"
            if arg.type != TypeHint.NO_TYPEHINT:
                arg_text += f": {arg.type}"
            arg_default = self.defaults[arg_i].value
            if arg_default != TypeHint.NO_DEFAULT:
                arg_text += f" = {arg_default}"
            args.append(arg_text)
            arg_i += 1

        if arg := self.vararg:
            arg_text = f"*{arg.name}"
            if arg.type != TypeHint.NO_TYPEHINT:
                arg_text += f": {arg.type}"
            args.append(arg_text)

        if len(self.kwonlyargs) > 0:
            # 加关键字参数分割符 *
            args.append("*")
            for arg, kw_default in zip(self.kwonlyargs, self.kw_defaults):
                arg_text = f"{arg.name}"
                if arg.type != TypeHint.NO_TYPEHINT:
                    arg_text += f": {arg.type}"
                if kw_default.value != TypeHint.NO_DEFAULT:
                    arg_text += f" = {kw_default.value}"
                args.append(arg_text)

        if self.kwarg is not None:
            arg_text = f"**{self.kwarg.name}"
            if self.kwarg.type != TypeHint.NO_TYPEHINT:
                arg_text += f": {self.kwarg.type}"
            args.append(arg_text)

        """魔法方法"""
        if self.name in self.magic_methods:
            if len(args) == 2:
                md += f"`{args[0]} {self.magic_methods[self.name]} {args[1]}"
            elif len(args) == 1:
                md += f"`{self.magic_methods[self.name]} {args[0]}"
            if self.return_ != TypeHint.NO_RETURN:
                md += f" => {self.return_}"
        else:
            md += f"`{self.name}("  # code start
            md += ", ".join(args) + ")"
            if self.return_ != TypeHint.NO_RETURN:
                md += f" -> {self.return_}"

        md += "`\n\n"  # code end

        """此处预留docstring"""
        if self.docs is not None:
            md += f"\n{self.docs.markdown(lang, indent)}\n"
        else:
            pass
        # 源码展示
        if kwargs.get("bu", None):
            # 源码链接
            self.module_file_path = self.module_file_path.replace("\\", "/")
            origin_url = kwargs.get("bu") + f"{self.module_file_path}#L{self.lineno}"
            # a_tag = f"\n\n[{get_text(lang, 'view_on_github')}]({origin_url})"
            a_tag = f"<a href='{origin_url}' target='_blank'>{get_text(lang, 'view_on_github')}</a>"
            or_and_a = f" {get_text(lang, 'or')} {a_tag}"
        else:
            a_tag = ""
            or_and_a = ""
        md += PREFIX + f"\n<details>\n<summary> <b>{get_text(lang, 'src')}</b>{or_and_a}</summary>\n\n```python\n{self.src}\n```\n</details>\n\n"

        return md

    def complete_default_args(self):
        """
        补全位置参数默认值，用无默认值插入
        Returns:

        """
        num = len(self.args) + len(self.posonlyargs) - len(self.defaults)
        self.defaults = [ConstantNode(value=TypeHint.NO_DEFAULT) for _ in range(num)] + self.defaults

    def __str__(self):
        return f"def {self.name}({', '.join([f'{arg.name}: {arg.type} = {arg.default}' for arg in self.args])}) -> {self.return_}"


class ClassNode(BaseModel):
    """
    ClassNode is a pydantic model that represents a class.
    Attributes:
        name: str
            The name of the class.
        docs: str = ""
            The docstring of the class.
        attrs: list[AttrNode] = []
            The attributes of the class.
        methods: list[MethodNode] = []
            The methods of the class.
        inherits: list["ClassNode"] = []
            The classes that the class inherits from
    """
    name: str
    docs: Optional[Docstring] = None
    attrs: list[AttrNode] = []
    methods: list[FunctionNode] = []
    inherits: list[str] = []

    def markdown(self, lang: str, **kwargs) -> str:
        """
        返回类的markdown文档
        Args:
            lang: str
                The language of the
        Returns:
            markdown style document
        """
        hidden_methods = [
                "__str__",
                "__repr__",
        ]
        md = ""
        md += f"### ***{kwargs.get('cd', 'class')}*** `{self.name}"
        if len(self.inherits) > 0:
            md += f"({', '.join([cls for cls in self.inherits])})"
        md += "`\n"
        for method in self.methods:
            if method.name in hidden_methods:
                continue
            md += method.markdown(lang, 2, **kwargs)
        for attr in self.attrs:
            if attr.type == TypeHint.NO_TYPEHINT:
                md += f"#### ***{kwargs.get('ad', get_text(lang, 'docstring.attribute'))}*** `{attr.name} = {attr.value}`\n\n"
            else:
                md += f"#### ***{kwargs.get('ad', get_text(lang, 'docstring.attribute'))}*** `{attr.name}: {attr.type} = {attr.value}`\n\n"

        return md



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/7 上午12:42
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : segment.py
@Software: PyCharm
"""

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .point import Point3  # type: ignore
    from .vector import Vector3  # type: ignore


class Segment3:
    def __init__(self, p1: "Point3", p2: "Point3"):
        """
        三维空间中的线段。
        :param p1:
        :param p2:
        """
        self.p1 = p1
        self.p2 = p2

        """方向向量"""
        self.direction = self.p2 - self.p1
        """长度"""
        self.length = self.direction.length
        """中心点"""
        self.midpoint = Point3((self.p1.x + self.p2.x) / 2, (self.p1.y + self.p2.y) / 2, (self.p1.z + self.p2.z) / 2)

    def __repr__(self):
        return f"Segment3({self.p1}, {self.p2})"

    def __str__(self):
        return f"Segment3({self.p1} -> {self.p2})"



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/26 上午6:29
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : angle.py
@Software: PyCharm
"""
import math
from typing import overload

from .const import PI  # type: ignore
from .utils import approx


class Angle:
    ...


class AnyAngle(Angle):
    def __init__(self, value: float, is_radian: bool = False):
        """
        任意角度。
        Args:
            value: 角度或弧度值
            is_radian: 是否为弧度，默认为否
        """
        if is_radian:
            self.radian = value
        else:
            self.radian = value * PI / 180

    @property
    def complementary(self) -> 'AnyAngle':
        """
        余角：两角的和为90°。
        Returns:
            余角
        """
        return AnyAngle(PI / 2 - self.minimum_positive.radian, is_radian=True)

    @property
    def supplementary(self) -> 'AnyAngle':
        """
        补角：两角的和为180°。
        Returns:
            补角
        """
        return AnyAngle(PI - self.minimum_positive.radian, is_radian=True)

    @property
    def degree(self) -> float:
        """
        角度。
        Returns:
            弧度
        """
        return self.radian * 180 / PI

    @property
    def minimum_positive(self) -> 'AnyAngle':
        """
        最小正角。
        Returns:
            最小正角度
        """
        return AnyAngle(self.radian % (2 * PI))

    @property
    def maximum_negative(self) -> 'AnyAngle':
        """
        最大负角。
        Returns:
            最大负角度
        """
        return AnyAngle(-self.radian % (2 * PI), is_radian=True)

    @property
    def sin(self) -> float:
        """
        正弦值。
        Returns:
            正弦值
        """
        return math.sin(self.radian)

    @property
    def cos(self) -> float:
        """
        余弦值。
        Returns:
            余弦值
        """
        return math.cos(self.radian)

    @property
    def tan(self) -> float:
        """
        正切值。
        Returns:
            正切值
        """
        return math.tan(self.radian)

    @property
    def cot(self) -> float:
        """
        余切值。
        Returns:
            余切值
        """
        return 1 / math.tan(self.radian)

    @property
    def sec(self) -> float:
        """
        正割值。
        Returns:
            正割值
        """
        return 1 / math.cos(self.radian)

    @property
    def csc(self) -> float:
        """
        余割值。
        Returns:
            余割值
        """
        return 1 / math.sin(self.radian)

    def __add__(self, other: 'AnyAngle') -> 'AnyAngle':
        return AnyAngle(self.radian + other.radian, is_radian=True)

    def __eq__(self, other):
        return approx(self.radian, other.radian)

    def __sub__(self, other: 'AnyAngle') -> 'AnyAngle':
        return AnyAngle(self.radian - other.radian, is_radian=True)

    def __mul__(self, other: float) -> 'AnyAngle':
        return AnyAngle(self.radian * other, is_radian=True)

    def __repr__(self):
        return f"AnyAngle({self.radian}, is_radian=True)"

    def __str__(self):
        return f"AnyAngle({self.degree}° or {self.radian} rad)"

    @overload
    def __truediv__(self, other: float) -> 'AnyAngle':
        ...

    @overload
    def __truediv__(self, other: 'AnyAngle') -> float:
        ...

    def __truediv__(self, other):
        if isinstance(other, AnyAngle):
            return self.radian / other.radian
        return AnyAngle(self.radian / other, is_radian=True)



# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/6 下午12:57
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : __init__.py.py
@Software: PyCharm
"""
from .angle import AnyAngle
from .const import *
from .equation import CurveEquation
from .line import Line3
from .plane import Plane3
from .point import Point3
from .segment import Segment3
from .vector import Vector3
